@@ -1,4 +1,4 @@
-{-# LANGUAGE TupleSections, BangPatterns, ConstraintKinds #-}
+{-# LANGUAGE TupleSections, ConstraintKinds #-}
 
 -- | This module extends "Data.List" with extra functions of a similar nature.
 --   The package also exports the existing "Data.List" functions.
@@ -28,10 +28,10 @@ module Data.List.Extra(
     nubOn, groupOn,
     nubSort, nubSortBy, nubSortOn,
     maximumOn, minimumOn,
-    disjoint, allSame, anySame,
-    repeatedly, for, firstJust,
+    disjoint, disjointOrd, disjointOrdBy, allSame, anySame,
+    repeatedly, firstJust,
     concatUnzip, concatUnzip3,
-    zipFrom, zipWithFrom,
+    zipFrom, zipWithFrom, zipWithLongest,
     replace, merge, mergeBy,
     ) where
 
@@ -59,13 +59,6 @@ repeatedly f as = b : repeatedly f as'
     where (b, as') = f as
 
 
--- | /DEPRECATED/ Use @flip map@ directly, since this function clashes with @Data.Traversable.for@.
---
---   Flipped version of 'map'.
-{-# DEPRECATED for "Use flip map directly, since this function clashes with Data.Traversable.for" #-}
-for :: [a] -> (a -> b) -> [b]
-for = flip map
-
 -- | Are two lists disjoint, with no elements in common.
 --
 -- > disjoint [1,2,3] [4,5] == True
@@ -73,6 +66,33 @@ for = flip map
 disjoint :: Eq a => [a] -> [a] -> Bool
 disjoint xs = null . intersect xs
 
+-- | /O((m+n) log m), m <= n/. Are two lists disjoint, with no elements in common.
+--
+-- @disjointOrd@ is more strict than `disjoint`. For example, @disjointOrd@ cannot
+-- terminate if both lists are inifite, while `disjoint` can.
+--
+-- > disjointOrd [1,2,3] [4,5] == True
+-- > disjointOrd [1,2,3] [4,1] == False
+disjointOrd :: Ord a => [a] -> [a] -> Bool
+disjointOrd = disjointOrdBy compare
+
+-- | A version of 'disjointOrd' with a custom predicate.
+--
+-- > disjointOrdBy (compare `on` (`mod` 7)) [1,2,3] [4,5] == True
+-- > disjointOrdBy (compare `on` (`mod` 7)) [1,2,3] [4,8] == False
+disjointOrdBy :: (a -> a -> Ordering) -> [a] -> [a] -> Bool
+disjointOrdBy cmp xs ys
+    | shorter xs ys = go xs ys
+    | otherwise = go ys xs
+  where
+    shorter _ [] = False
+    shorter [] _ = True
+    shorter (_:xs) (_:ys) = shorter xs ys
+
+    go xs = not . any (\a -> memberRB cmp a tree)
+      where
+        tree = foldl' (flip (insertRB cmp)) E xs
+
 -- | Is there any element which occurs more than once.
 --
 -- > anySame [1,1,2] == True
@@ -211,20 +231,18 @@ splitAtEnd i xs = f xs (drop i xs)
 --   Never truncates the output - raises an error if the enumeration runs out.
 --
 -- > \i xs -> zip [i..] xs == zipFrom i xs
--- > zipFrom False [1..3] == undefined
-zipFrom :: (Partial, Enum a) => a -> [b] -> [(a, b)]
+-- > zipFrom False [1..3] == [(False,1),(True, 2)]
+zipFrom :: Enum a => a -> [b] -> [(a, b)]
 zipFrom = zipWithFrom (,)
 
 -- | 'zipFrom' generalised to any combining operation.
 --   Never truncates the output - raises an error if the enumeration runs out.
 --
 -- > \i xs -> zipWithFrom (,) i xs == zipFrom i xs
-zipWithFrom :: (Partial, Enum a) => (a -> b -> c) -> a -> [b] -> [c]
-zipWithFrom f a xs = go a xs
-    where
-        -- if we aren't strict in the accumulator, it's highly like to be a space leak
-        go !a [] = []
-        go !a (x:xs) = f a x : go (succ a) xs
+zipWithFrom :: Enum a => (a -> b -> c) -> a -> [b] -> [c]
+-- would love to deforest the intermediate [a..] list
+-- but would require Bounded and Eq as well, so better go for simplicit
+zipWithFrom f a = zipWith f [a..]
 
 
 -- | A merging of 'unzip' and 'concat'.
@@ -742,39 +760,59 @@ nubOrdBy cmp xs = f E xs
 ---------------------------------------------------------------------
 -- OKASAKI RED BLACK TREE
 -- Taken from https://www.cs.kent.ac.uk/people/staff/smk/redblack/Untyped.hs
+-- But with the Color = R|B fused into the tree
 
-data Color = R | B deriving Show
-data RB a = E | T Color (RB a) a (RB a) deriving Show
+data RB a = E | T_R (RB a) a (RB a) | T_B (RB a) a (RB a) deriving Show
 
 {- Insertion and membership test as by Okasaki -}
 insertRB :: (a -> a -> Ordering) -> a -> RB a -> RB a
-insertRB cmp x s =
-    T B a z b
+insertRB cmp x s = case ins s of
+    T_R a z b -> T_B a z b
+    x -> x
     where
-    T _ a z b = ins s
-    ins E = T R E x E
-    ins s@(T B a y b) = case cmp x y of
-        LT -> balance (ins a) y b
-        GT -> balance a y (ins b)
+    ins E = T_R E x E
+    ins s@(T_B a y b) = case cmp x y of
+        LT -> lbalance (ins a) y b
+        GT -> rbalance a y (ins b)
         EQ -> s
-    ins s@(T R a y b) = case cmp x y of
-        LT -> T R (ins a) y b
-        GT -> T R a y (ins b)
+    ins s@(T_R a y b) = case cmp x y of
+        LT -> T_R (ins a) y b
+        GT -> T_R a y (ins b)
         EQ -> s
 
 memberRB :: (a -> a -> Ordering) -> a -> RB a -> Bool
 memberRB cmp x E = False
-memberRB cmp x (T _ a y b) = case cmp x y of
+memberRB cmp x (T_R a y b) = case cmp x y of
+    LT -> memberRB cmp x a
+    GT -> memberRB cmp x b
+    EQ -> True
+memberRB cmp x (T_B a y b) = case cmp x y of
     LT -> memberRB cmp x a
     GT -> memberRB cmp x b
     EQ -> True
 
 {- balance: first equation is new,
    to make it work with a weaker invariant -}
-balance :: RB a -> a -> RB a -> RB a
-balance (T R a x b) y (T R c z d) = T R (T B a x b) y (T B c z d)
-balance (T R (T R a x b) y c) z d = T R (T B a x b) y (T B c z d)
-balance (T R a x (T R b y c)) z d = T R (T B a x b) y (T B c z d)
-balance a x (T R b y (T R c z d)) = T R (T B a x b) y (T B c z d)
-balance a x (T R (T R b y c) z d) = T R (T B a x b) y (T B c z d)
-balance a x b = T B a x b
+lbalance, rbalance :: RB a -> a -> RB a -> RB a
+lbalance (T_R a x b) y (T_R c z d) = T_R (T_B a x b) y (T_B c z d)
+lbalance (T_R (T_R a x b) y c) z d = T_R (T_B a x b) y (T_B c z d)
+lbalance (T_R a x (T_R b y c)) z d = T_R (T_B a x b) y (T_B c z d)
+lbalance a x b = T_B a x b
+rbalance (T_R a x b) y (T_R c z d) = T_R (T_B a x b) y (T_B c z d)
+rbalance a x (T_R b y (T_R c z d)) = T_R (T_B a x b) y (T_B c z d)
+rbalance a x (T_R (T_R b y c) z d) = T_R (T_B a x b) y (T_B c z d)
+rbalance a x b = T_B a x b
+
+
+-- | Like 'zipWith', but keep going to the longest value. The function
+--   argument will always be given at least one 'Just', and while both
+--   lists have items, two 'Just' values.
+--
+-- > zipWithLongest (,) "a" "xyz" == [(Just 'a', Just 'x'), (Nothing, Just 'y'), (Nothing, Just 'z')]
+-- > zipWithLongest (,) "a" "x" == [(Just 'a', Just 'x')]
+-- > zipWithLongest (,) "" "x" == [(Nothing, Just 'x')]
+zipWithLongest :: (Maybe a -> Maybe b -> c) -> [a] -> [b] -> [c]
+zipWithLongest f [] [] = []
+zipWithLongest f (x:xs) (y:ys) = f (Just x) (Just y) : zipWithLongest f xs ys
+zipWithLongest f [] ys = map (f Nothing . Just) ys
+zipWithLongest f xs [] = map ((`f` Nothing) . Just) xs
